searchData={"items":[{"type":"module","title":"LibEcto","doc":"LibEcto is a simple wrapper for [ecto](https://hexdocs.pm/ecto/Ecto.html), make it much easier for daily use.","ref":"LibEcto.html"},{"type":"module","title":"Why LibEcto - LibEcto","doc":"Ecto is a great library, but it's a little bit verbose for daily use.\n\nFor example, imaging you have a schema like this:\n\n```Elixir\ndefmodule Sample.Schema do\n  use Ecto.Schema\n  import Ecto.Changeset\n\n  @primary_key {:id, LibEcto.KsuidType, autogenerate: true}\n  schema \"test\" do\n    field :name, :string\n    field :value, :string\n\n    timestamps()\n  end\n\n  def changeset(m, params) do\n    m\n    |> cast(params, [:name, :value])\n  end\nend\n```\n\nFor most common use case, you need to write a lot of boilerplate code to do simple CRUD operation:\n\n```Elixir\ndefmodule Sample.DB do\n\n  alias Sample.Schema\n  alias Sample.Repo\n  import Ecto.Changeset\n\n  def insert_one(params) do\n    Schema.changeset(%Schema{}, params)\n    |> Repo.insert()\n  end\n\n  def update_one(m, params) do\n    m\n    |> change(params)\n    |> Repo.update()\n  end\n\n  def delete_one(m) do\n    Repo.delete(m)\n  end\n\n\n  def get_by_id(id) do\n    Repo.get(Schema, id, select: [:id, :name, :value])\n  end\n\n  def get_by_id_array(id_array) do\n    Repo.all(from m in Schema, where: m.id in ^id_array, select: [:id, :name, :value])\n  end\n\n  def get_by_name(name) do\n    Repo.get_by(Schema, name: name, select: [:id, :name, :value])\n  end\n\n  #... more boilerplate code\n  # - get by name array\n  # - get by page\n  # - get by prefix\n\nend\n```\n\nBut!!!!!! With LibEcto, you can code like this:\n\n```Elixir\ndefmodule Sample.DB do\n    use LibEcto,\n      repo: Sample.Repo,\n      schema: Sample.Schema,\n      columns: [\n        :id,\n        :name,\n        :value\n      ],\n      filters: [\n        :id,\n        :name\n      ]\n\n    def filter(:id, dynamic, %{\"id\" => value}) when is_bitstring(value),\n      do: {:ok, dynamic([m], ^dynamic and m.id == ^value)}\n\n    def filter(:id, dynamic, %{\"id\" => value}) when is_list(value),\n      do: {:ok, dynamic([m], ^dynamic and m.id in ^value)}\n\n    def filter(:name, dynamic, %{\"name\" => value}) when is_bitstring(value),\n      do: {:ok, dynamic([m], ^dynamic and m.name == ^value)}\n\n    def filter(:name, dynamic, %{\"name\" => {\"like\", value}}) when is_bitstring(value),\n      do: {:ok, dynamic([m], ^dynamic and like(m.name, ^value))}\n\n    def filter(:name, dynamic, %{\"name\" => value}) when is_list(value),\n      do: {:ok, dynamic([m], ^dynamic and m.name in ^value)}\n\n    def filter(_, dynamic, _), do: {:ok, dynamic}\n\n\n    def init_filter, do: dynamic([m], true)\n\n\n    # you can use ecto's ability to build complicate query or update or transaction if GenericDB can't satisfy your need\n    def other_complicated_query_or_update() do\n      # do something\n    end\nend\n```\n\nLibEcto will generate all the boilerplate code for you, and you can focus on your business logic:\n\n\n```Elixir\niex> Sample.DB.create_one(%{name: \"test\", value: \"testv\"})\n{:ok, %Simple.Schema{id: \"2JIebKci1ZgKenvhllJa3PMbydB\", name: \"test\", value: \"testv\"}}\n\niex> Sample.DB.get_one(%{\"name\" => \"test\"})\n{:ok, %Simple.Schema{id: \"2JIebKci1ZgKenvhllJa3PMbydB\", name: \"test\", value: \"testv\"}}\n\niex> Sample.DB.get_one(%{\"name\" => \"not-exists\"})\n{:ok, nil}\n\niex> Sample.DB.get_one!(%{\"name\" => \"not-exists\"})\n{:error, 404}\n\niex> {:ok, m} = Sample.DB.get_one(%{\"name\" => \"test\"})\niex> Sample.DB.update_one(m, name: \"test2\")\n{:ok, %Simple.Schema{id: \"2JIebKci1ZgKenvhllJa3PMbydB\", name: \"test2\", value: \"testv\"}}\n```\nAll supported functions:\n\n- create_one/1\n- get_one/1\n- get_one!/1\n- get_all/2\n- get_limit/4\n- count/1\n- exists?/1\n- get_by_page/5\n- update_one/2\n- delete_one/1\n- delete_all/1\n\nFor more usage details, please check the test cases, which covers all the supported functions.","ref":"LibEcto.html#module-why-libecto"},{"type":"module","title":"V2 - LibEcto","doc":"V2 is a complete rewrite of LibEcto, it's much more powerful and flexible. It simply breaks down the original complex macros into more reasonable ones. The only difference with LibEcto is the DB layer, Here's how to use it:\n\n```Elixir\n  defmodule Test.DB do\n    @moduledoc false\n    use LibEctoV2\n\n    @repo Repo\n    @schema Test.Schema\n    @columns [:id, :name, :value]\n    @filters [:id, :name]\n\n    def filter(:id, dynamic, %{\"id\" => value}) when is_bitstring(value),\n      do: {:ok, dynamic([m], ^dynamic and m.id == ^value)}\n\n    def filter(:id, dynamic, %{\"id\" => value}) when is_list(value), do: {:ok, dynamic([m], ^dynamic and m.id in ^value)}\n\n    def filter(:name, dynamic, %{\"name\" => value}) when is_bitstring(value),\n      do: {:ok, dynamic([m], ^dynamic and m.name == ^value)}\n\n    def filter(:name, dynamic, %{\"name\" => {\"like\", value}}) when is_bitstring(value),\n      do: {:ok, dynamic([m], ^dynamic and like(m.name, ^value))}\n\n    def filter(:name, dynamic, %{\"name\" => value}) when is_list(value),\n      do: {:ok, dynamic([m], ^dynamic and m.name in ^value)}\n\n    def filter(_, dynamic, _), do: {:ok, dynamic}\n\n    def init_filter, do: dynamic([m], m.removed_at == 0)\n  end\n```","ref":"LibEcto.html#module-v2"},{"type":"module","title":"Installation - LibEcto","doc":"The package can be installed by adding `lib_ecto` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:lib_ecto, \"~> 0.3\"}\n  ]\nend\n```","ref":"LibEcto.html#module-installation"},{"type":"module","title":"Test - LibEcto","doc":"Test cases use [ecto_sqlite3](https://github.com/elixir-sqlite/ecto_sqlite3) as database.\n\n\n```bash\nFinished in 1.5 seconds (0.00s async, 1.5s sync)\n17 tests, 0 failures\n\nRandomized with seed 336538\n\nGenerating cover results ...\n\nPercentage | Module\n-----------|--------------------------\n    40.00% | LibEcto.KsuidType\n    78.95% | LibEcto.Ksuid\n   100.00% | LibEcto\n   100.00% | LibEcto.Base62\n-----------|--------------------------\n    76.19% | Total\n\nCoverage test failed, threshold not met:\n\n    Coverage:   76.19%\n    Threshold:  90.00%\n\nGenerated HTML coverage results in \"cover\" directory\n```","ref":"LibEcto.html#module-test"},{"type":"module","title":"LibEcto.Base62","doc":"Provides functions for encoding and decoding data using the Base62 encoding scheme.\n\nBase62 is a binary-to-text encoding scheme that represents binary data in an ASCII string format.\nIt uses a set of 62 characters (0-9, a-z, A-Z) to represent binary data in a way\nthat is safe for use in URLs and other contexts where certain characters may be reserved or have special meaning.\n\nThis module provides functions for encoding and decoding binary data or integers\nusing the Base62 encoding scheme.\nThe `encode/1` and `decode!/1` functions are the primary entry points for encoding and decoding data, respectively.\n\nExample usage:\n\n    ```\n    iex> Base62.encode(\"hello world\")\n    \"dGhpcyBpcyBhIHRlc3Q=\"\n    iex> Base62.decode!(\"dGhpcyBpcyBhIHRlc3Q=\")\n    \"hello world\"\n    ```","ref":"LibEcto.Base62.html"},{"type":"function","title":"LibEcto.Base62.bin_to_integer/1","doc":"","ref":"LibEcto.Base62.html#bin_to_integer/1"},{"type":"function","title":"LibEcto.Base62.decode!/1","doc":"","ref":"LibEcto.Base62.html#decode!/1"},{"type":"function","title":"LibEcto.Base62.encode/1","doc":"","ref":"LibEcto.Base62.html#encode/1"},{"type":"module","title":"LibEcto.Ksuid","doc":"ksuid is a zero dependency Elixir library for generating and parsing KSUIDs.\nRead more about ksuid [here](https://segment.com/blog/a-brief-history-of-the-uuid/)\n\nksuid stands for K-Sortable Unique Identifier. It's a way to generate globally unique\nIDs which are partially chronologically sortable.","ref":"LibEcto.Ksuid.html"},{"type":"module","title":"How To - LibEcto.Ksuid","doc":"```elixir\n\niex> LibEcto.Ksuid.generate()\n\"0p9kxW1vWavpdq7VSgbv8piY0nr\"\n\niex> LibEcto.Ksuid.parse(\"0p9kxW1vWavpdq7VSgbv8piY0nr\")\n{\n  :ok,\n  %DateTime{calendar: Calendar.ISO, day: 9, hour: 14, microsecond: {0, 0},\n    minute: 52, month: 6, second: 34, std_offset: 0, time_zone: \"Etc/UTC\",\n    utc_offset: 0, year: 2017, zone_abbr: \"UTC\"},\n  <<166, 90, 80, 117, 89, 88, 196, 168, 113, 163, 157, 217, 224, 51, 151, 227>>\n }\n\n```","ref":"LibEcto.Ksuid.html#module-how-to"},{"type":"module","title":"Example Ecto Usage - LibEcto.Ksuid","doc":"#","ref":"LibEcto.Ksuid.html#module-example-ecto-usage"},{"type":"module","title":"Ecto Type - LibEcto.Ksuid","doc":"```elixir\ndefmodule LibEcto.KsuidType do\n  @moduledoc \"\"\"\n  types for ksuid\n  uses string/varchar as storage type.\n  \"\"\"\n\n  @behaviour Ecto.Type\n  alias LibEcto.Ksuid\n\n  def type, do: :string\n\n  def cast(ksuid) when is_binary(ksuid), do: {:ok, ksuid}\n  def cast(_), do: :error\n\n  @doc \"\"\"\n  Same as `cast/1` but raises `Ecto.CastError` on invalid arguments.\n  \"\"\"\n  def cast!(value) do\n    case cast(value) do\n      {:ok, ksuid} -> ksuid\n      :error -> raise Ecto.CastError, type: __MODULE__, value: value\n    end\n  end\n\n  def load(ksuid), do: {:ok, ksuid}\n\n  def dump(binary) when is_binary(binary), do: {:ok, binary}\n  def dump(_), do: :error\n\n  # Callback invoked by autogenerate fields - this is all that really matters\n  # just passing around the binary otherwise.\n  @doc false\n  def autogenerate, do: Ksuid.generate()\nend\n\n```\n\n#","ref":"LibEcto.Ksuid.html#module-ecto-type"},{"type":"module","title":"Usage in a Schema - LibEcto.Ksuid","doc":"`:inserted_at` is a virtual field that can be derived/loaded from the autogenerated `:id`\n\n```elixir\ndefmodule TestSchema do\n  use Ecto.Schema\n\n  @primary_key {:id, KsuidType, autogenerate: true}\n  schema \"test\" do\n    field :name, :string\n    field :inserted_at, :utc_datetime, virtual: true\n  end\n\n  def inserted_at(%TestSchema{id: ksuid} = row) do\n     {:ok, time_stamp, _} = LibEcto.Ksuid.parse(ksuid)\n     %TestSchema{row | inserted_at: time_stamp}\n  end\n  \nend\n```\n##","ref":"LibEcto.Ksuid.html#module-usage-in-a-schema"},{"type":"module","title":"Migration - LibEcto.Ksuid","doc":"Create `:id` as `:bytea`, and primary key - similary to usage with `:binary_id`\n\n```elixir\n  def change do\n     create table(:test, primary_key: false) do\n      add :id, :string, primary_key: true, size: 27\n      add :name, :text\n     \n    end\n  end\n```\n\n\n#","ref":"LibEcto.Ksuid.html#module-migration"},{"type":"module","title":"Attention!!!! - LibEcto.Ksuid","doc":"Some databases(like MySQL) have a default case-insensitive comparison for binary. If you use ksuid as the primary key and need to compare or sort ids, you need to use the SQL binary() function.\n\n```Elixir\nquery = from t in TestSchema, where: fragment(\"binary(?)\", t.id) > ^id\n\nrepo.all(query, order_by: [desc: fragment(\"binary(?)\", t.id)])\n```","ref":"LibEcto.Ksuid.html#module-attention"},{"type":"function","title":"LibEcto.Ksuid.generate/1","doc":"This method returns a 20 byte Ksuid which has 4 bytes as timestamp\nand 16 bytes of crypto string bytes.","ref":"LibEcto.Ksuid.html#generate/1"},{"type":"function","title":"Examples - LibEcto.Ksuid.generate/1","doc":"iex> Ksuid.generate()\n    \"0KZi94b2fnVzpGi60FoZgXIvUtYy\"","ref":"LibEcto.Ksuid.html#generate/1-examples"},{"type":"function","title":"LibEcto.Ksuid.parse/1","doc":"","ref":"LibEcto.Ksuid.html#parse/1"},{"type":"module","title":"LibEcto.KsuidType","doc":"types for ksuid\nuses string/varchar as storage type.","ref":"LibEcto.KsuidType.html"},{"type":"module","title":"Example - LibEcto.KsuidType","doc":"```Elixir\ndefmodule TestSchema do\n  use Ecto.Schema\n  alias LibEcto.KsuidType\n\n  @primary_key {:id, KsuidType, autogenerate: true}\n  schema \"test\" do\n    field :name, :string\n    field :inserted_at, :utc_datetime, virtual: true\n  end\n\n  def inserted_at(%TestSchema{id: ksuid} = row) do\n     {:ok, time_stamp, _} = LibEcto.Ksuid.parse(ksuid)\n     %TestSchema{row | inserted_at: time_stamp}\n  end\nend\n```","ref":"LibEcto.KsuidType.html#module-example"},{"type":"module","title":"BE CAREFUL!!!! - LibEcto.KsuidType","doc":"If you are using ksuid as your primary key, it is important to declare the id type as binary when creating your table.\nFailing to do so may result in problems with sorting and comparison.\n\n```SQL\nCREATE TABLE test_table (\n  id VARCHAR(20) BINARY PRIMARY KEY,\n  name VARCHAR(50)\n);\n```","ref":"LibEcto.KsuidType.html#module-be-careful"},{"type":"function","title":"LibEcto.KsuidType.cast/1","doc":"","ref":"LibEcto.KsuidType.html#cast/1"},{"type":"function","title":"LibEcto.KsuidType.cast!/1","doc":"Same as `cast/1` but raises `Ecto.CastError` on invalid arguments.","ref":"LibEcto.KsuidType.html#cast!/1"},{"type":"function","title":"LibEcto.KsuidType.dump/1","doc":"","ref":"LibEcto.KsuidType.html#dump/1"},{"type":"function","title":"LibEcto.KsuidType.embed_as/1","doc":"","ref":"LibEcto.KsuidType.html#embed_as/1"},{"type":"function","title":"LibEcto.KsuidType.equal?/2","doc":"","ref":"LibEcto.KsuidType.html#equal?/2"},{"type":"function","title":"LibEcto.KsuidType.load/1","doc":"","ref":"LibEcto.KsuidType.html#load/1"},{"type":"function","title":"LibEcto.KsuidType.type/0","doc":"","ref":"LibEcto.KsuidType.html#type/0"},{"type":"module","title":"LibEctoV2","doc":"V2 is a complete rewrite of LibEcto, it's much more powerful and flexible. It simply breaks down the original complex macros into more reasonable ones. Here's how to use it:\n\n#","ref":"LibEctoV2.html"},{"type":"module","title":"Setup your Schema - LibEctoV2","doc":"```Elixir\n   defmodule Test.Schema do\n    @moduledoc false\n    use Ecto.Schema\n\n    import Ecto.Changeset\n\n    @primary_key {:id, KsuidType, autogenerate: true}\n    schema \"test\" do\n      field(:name, :string)\n      field(:value, :string)\n      field(:removed_at, :integer)\n    end\n\n    def changeset(m, params) do\n      cast(m, params, [:name, :value])\n    end\n  end\n```\n\n#","ref":"LibEctoV2.html#module-setup-your-schema"},{"type":"module","title":"use LibEctoV2 to setup DB layer - LibEctoV2","doc":"```Elixir\n  defmodule Test.DB do\n    @moduledoc false\n    use LibEctoV2\n\n    @repo Repo\n    @schema Test.Schema\n    @columns [:id, :name, :value]\n    @filters [:id, :name]\n\n    def filter(:id, dynamic, %{\"id\" => value}) when is_binary(value),\n      do: {:ok, dynamic([m], ^dynamic and fragment(\"binary(?)\", m.id) == ^value)}\n\n    def filter(:id, dynamic, %{\"id\" => value}) when is_list(value), do: {:ok, dynamic([m], ^dynamic and m.id in ^value)}\n\n    def filter(:name, dynamic, %{\"name\" => value}) when is_binary(value),\n      do: {:ok, dynamic([m], ^dynamic and m.name == ^value)}\n\n    def filter(:name, dynamic, %{\"name\" => {\"like\", value}}) when is_binary(value),\n      do: {:ok, dynamic([m], ^dynamic and like(m.name, ^value))}\n\n    def filter(:name, dynamic, %{\"name\" => value}) when is_list(value),\n      do: {:ok, dynamic([m], ^dynamic and m.name in ^value)}\n\n    def filter(_, dynamic, _), do: {:ok, dynamic}\n\n    def init_filter, do: dynamic([m], m.removed_at == 0)\n  end\n```\n\n\n#","ref":"LibEctoV2.html#module-use-libectov2-to-setup-db-layer"},{"type":"module","title":"Enjoy it - LibEctoV2","doc":"```Elixir\niex> Sample.DB.create_one(%{name: \"test\", value: \"testv\"})\n{:ok, %Simple.Schema{id: \"2JIebKci1ZgKenvhllJa3PMbydB\", name: \"test\", value: \"testv\"}}\n\niex> Sample.DB.get_one(%{\"name\" => \"test\"})\n{:ok, %Simple.Schema{id: \"2JIebKci1ZgKenvhllJa3PMbydB\", name: \"test\", value: \"testv\"}}\n\niex> Sample.DB.get_one(%{\"name\" => \"not-exists\"})\n{:ok, nil}\n\niex> Sample.DB.get_one!(%{\"name\" => \"not-exists\"})\n{:error, 404}\n\niex> {:ok, m} = Sample.DB.get_one(%{\"name\" => \"test\"})\niex> Sample.DB.update_one(m, name: \"test2\")\n{:ok, %Simple.Schema{id: \"2JIebKci1ZgKenvhllJa3PMbydB\", name: \"test2\", value: \"testv\"}}\n```","ref":"LibEctoV2.html#module-enjoy-it"},{"type":"function","title":"LibEctoV2.build_condition/5","doc":"","ref":"LibEctoV2.html#build_condition/5"},{"type":"function","title":"LibEctoV2.generate_get_many_funcs/4","doc":"","ref":"LibEctoV2.html#generate_get_many_funcs/4"},{"type":"function","title":"LibEctoV2.generate_get_one_funcs/4","doc":"","ref":"LibEctoV2.html#generate_get_one_funcs/4"},{"type":"function","title":"LibEctoV2.generate_other_funcs/3","doc":"","ref":"LibEctoV2.html#generate_other_funcs/3"},{"type":"function","title":"LibEctoV2.generate_types/1","doc":"","ref":"LibEctoV2.html#generate_types/1"},{"type":"function","title":"LibEctoV2.generate_write_funcs/3","doc":"","ref":"LibEctoV2.html#generate_write_funcs/3"}],"content_type":"text/markdown"}